//
// Fragment shader for computing vertex positions & normals for R2VB water rendering demo
//

#define		SIZE		16.0
#define		TEX_SIZE	128.0
#define		SCALE1		2.073
#define		SCALE2		4.235
#define		SCALE3		8.0973

uniform	sampler2D	h1Map;						// height maps
uniform	sampler2D	h2Map;
uniform	sampler2D	h3Map;
uniform	sampler2D	h1Map2;						// next frame height maps
uniform	sampler2D	h2Map2;
uniform	sampler2D	h3Map2;
uniform	float		time;
uniform float		amplitude;

vec3	clr2vec3 ( const vec3 clr )
{
	return 2.0 * clr - vec3 ( 1.0 );
}

void	main ()
{										// deltas
	const vec3	dx  = vec3 ( 0.01, 0.0,  0.0 );
	const vec3	dy  = vec3 ( 0.0,  0.01, 0.0 );
	const vec2	vel = vec2 ( 0.02, 0.01 )*0.2;
	const vec2	b1   = vec2 ( 0.1074, 0.7083 );		// texCooord biases
	const vec2	b2   = vec2 ( 0.3571, 0.1144 );
	
										// compute height and delta height (in dx and dy)
	vec2	tex    = vec2  ( gl_TexCoord [0].xy + vel * time );
	vec2	tex2   = vec2  ( gl_TexCoord [0].xy + vel * time );
	vec2	tex3   = vec2  ( gl_TexCoord [0].xy + vel * time );
	float	t      = fract ( SCALE1 * time );
	float	t2     = fract ( SCALE2 * time );
	float	t3     = fract ( SCALE3 * time );
	vec4	h1     = mix   ( texture2D ( h1Map, SCALE1 * tex + b1 ),  texture2D ( h1Map2, SCALE1 * tex  + b1 ), t );
	vec4	h2     = mix   ( texture2D ( h2Map, SCALE2 * tex2 + b2 ), texture2D ( h2Map2, SCALE2 * tex2 + b2 ), t2 );
//	vec4	h3     = mix   ( texture2D ( h3Map, 4.0 * tex3 + b3 ), texture2D ( h1Map2, 4.0 * tex3 + b3 ), t3 );
	vec4	h      = 0.5*h1 + 0.25 * h2;//0.5 * h2;// + 0.25 * h3;
	vec3	n      = clr2vec3 ( h1.xyz ) + clr2vec3 ( h2.xyz );// + clr2vec3 ( h3 );
	
	gl_FragData [0] = vec4 ( gl_TexCoord [0].xy * 2.0 * SIZE - vec2 ( SIZE ), -2.0 + amplitude * h.w, 1.0 );
	gl_FragData [1] = vec4 ( normalize ( n ), 1.0 );
}
