//
// Rippled water fragment shader
//

uniform samplerCube envMap;
uniform float		time;
uniform float		amplitude;

varying vec3  n;
varying vec3  e;
varying	vec3  p;

uniform sampler3D 	noiseMap;

//
// takes values in [-1,1] range
//

vec3	turbulence ( const in vec3 p, const in float freqScale )
{
	float	sum = 0.0;
	vec3	t   = vec3 ( 0.0 );
	float	f   = 1.0;

	for ( int i = 0; i <= 3; i++ )
	{
		t   += abs ( 2.0 * texture3D ( noiseMap, f * p.xyz ).rgb - vec3 ( 1.0 ) ) / f;
		sum += 1.0 / f;
		f   *= freqScale;
	}
										// remap from [0,sum] to [-1,1]
	return 2.0 * t / sum - vec3 ( 1.0 );
}

void main (void)
{
	const vec4	c2 = 2.2 * vec4 ( 0.03, 0.15, 0.125, 1.0 );
	const vec4	c1 = 1.6 * vec4 ( 0.03, 0.2,  0.07, 1.0 );

    					// Compute reflection vector
	vec3	arg = 0.25*p + vec3 ( 0.04101, -0.0612149, 0.071109 ) * time * 0.7;
	vec3	ns  = vec3 ( turbulence ( arg, 2.17 ).xy, 1.0 );
  	vec3	en  = normalize ( e );
  	vec3	nn  = normalize ( n + amplitude * ns );
    vec3	r   = reflect   ( en, nn );
    
    					// Do a lookup into the environment map.
    vec4 envColor = textureCube ( envMap, r );

    					// calc fresnels term.  This allows a view dependant
    					// blend of reflection/refraction
    float fresnel = clamp ( abs ( dot ( en, nn ) ), 0.1, 0.9 );

	gl_FragColor = vec4 ( mix ( c1, c2, abs ( dot ( en, nn ) ) )*fresnel +  (1.0-fresnel) * envColor );
}